/*
 * Copyright "2011" Adam Warski and Amanj Sherwany
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
package net.java.loci;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.resolver.ArtifactResolutionException;
import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
import org.apache.maven.toolchain.ToolchainManager;
import org.apache.maven.toolchain.Toolchain;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.plugin.MojoExecutionException;
import org.codehaus.plexus.util.DirectoryScanner;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * This is a modified version of Checker framework's maven plugin,
 * I removed everything related to the Checker's framework and put Loci
 * stuff instead. The work also depends on Checker framework's maven plugin.
 * @author Adam Warski (adam at warski dot org)
 * @author Amanj Sherwany
 */
public class PathUtils {
	private final static String LOCI_GROUP_ID = "net.java.loci";
	private final static String LOCI_ARTIFACT_ID = "jsr308-all";

	
	/**
	 * Gets the path to the JSR308 jar.
	 * @param jsrVersion Version of JSR308 to use.
	 * @param artifactFactory Dependency.
	 * @param artifactResolver Dependency.
	 * @param remoteArtifactRepositories Dependency.
	 * @param localRepository Dependency.
	 * @return Path to the Loci jar.
	 * @throws MojoExecutionException
	 */
	public static String getJSRJar(String jsrVersion, ArtifactFactory artifactFactory,
										ArtifactResolver artifactResolver, List<?> remoteArtifactRepositories,
										ArtifactRepository localRepository) throws MojoExecutionException {
		Artifact jsrArtifact;
		try {
			jsrArtifact = artifactFactory.createExtensionArtifact(LOCI_GROUP_ID, LOCI_ARTIFACT_ID,
					VersionRange.createFromVersionSpec(jsrVersion));
		} catch (InvalidVersionSpecificationException e) {
			throw new MojoExecutionException("Wrong version: " + jsrVersion + " of JSR308 specifed.", e);
		}

		try {
			artifactResolver.resolve(jsrArtifact, remoteArtifactRepositories, localRepository);
		} catch (ArtifactResolutionException e) {
			throw new MojoExecutionException("Unable to find version " + jsrVersion + " of JSR308.", e);
		} catch (ArtifactNotFoundException e) {
			throw new MojoExecutionException("Unable to resolve version " + jsrVersion + " of JSR308.", e);
		}

		return jsrArtifact.getFile().getPath();
	}

	/**
	 * Gets the path to the given executable, looking it up in the toolchain.
	 * @param executable Name of the executable.
	 * @param toolchainManager Dependency.
	 * @param session Dependency.
	 * @return Path to the executable.
	 */
	public static String getExecutablePath(String executable, ToolchainManager toolchainManager, MavenSession session) {
		File execFile = new File(executable);
		if (execFile.exists()) {
			return execFile.getAbsolutePath();
		} else {
			Toolchain tc = toolchainManager.getToolchainFromBuildContext("jdk", session);

			if (tc != null) {
				executable = tc.findTool(executable);
			}
		}

		return executable;
	}

	/**
	 * Scans the given compile source roots for sources, taking into account the given includes and excludes.
	 * @param compileSourceRoots A list of source roots. 
	 * @param sourceIncludes Includes specification.
	 * @param sourceExcludes Excludes specification.
	 * @return A list of included sources from the given source roots.
	 */
	public static List<String> scanForSources(List<?> compileSourceRoots, Set<String> sourceIncludes, Set<String> sourceExcludes) {
		if (sourceIncludes.isEmpty()) {
			sourceIncludes.add("**/*.java");
		}

		List<String> sources = new ArrayList<String>();

		for (Object compileSourceRoot : compileSourceRoots) {
			File compileSourceRootFile = new File(compileSourceRoot.toString());
			String[] sourcesFromSourceRoot =
					scanForSources(compileSourceRootFile, sourceIncludes, sourceExcludes);

			for (String sourceFromSourceRoot : sourcesFromSourceRoot) {
				sources.add(new File(compileSourceRootFile, sourceFromSourceRoot).getAbsolutePath());
			}
		}

		return sources;
	}

	private static String[] scanForSources(File sourceDir, Set<String> sourceIncludes, Set<String> sourceExcludes) {
		DirectoryScanner ds = new DirectoryScanner();
		ds.setFollowSymlinks( true );
		ds.setBasedir( sourceDir );

		ds.setIncludes(sourceIncludes.toArray(new String[sourceIncludes.size()]));
		ds.setExcludes(sourceExcludes.toArray(new String[sourceExcludes.size()]));

		ds.addDefaultExcludes();

		ds.scan();

		return ds.getIncludedFiles();
	}
}
